/*
  File: IConnector.java

  History:
  Date       	Who				What
  May_19_2003  	poladian		Created public version
*/
package messaging;
/**
*	This interface is the root of all Connectors.
*
*   Generally speaking, there are 6 kinds of connectors: 2 for each component,
*   in particular, one for communication with each of the remaining two
*   components. ( e.g., Prism to EM connector, EM To Prism connector, etc)

*   For communicating with Suppliers, a single connector handles
*   communication with all the suppliers.

*   A connector provides ability to receive messages through IListener,
*   and ability to send messages Senders.  Thus, a connector will internally
*   manage one or more Listeners and one or more Senders.

*   IConnector extends the IMessageQueue interface and serves as a
*   temporary queue for the received messages.  Queue messages will be
*   eventually provided to callbacks for handling.  In order to call callback
*   methods, the connector is responsible for managing an internal thread pool.
*   Connector may choose to have a single thread for this purpose, a fixed
*   number of threads, or a flexible thread pool.

*   This interface provides two versions of the general send() methods.
*   One of the send methods allows
*   sending to the default destination, and the other allows specifying
*   destination on a per-invocation basis.

*   The implementing connectors are responsible for determining the default
*   destination, either through constructor parameters, or externally
*   specified configuration parameters (see aura.config file).

*   For the processing of the received messages, methods are provided for setting
*   a callback.  It is possible that a callback is not set at a given time.
*   In this case, the messages be stored in an internal queue.  Once a callback
*   is set, messages will be consumed by calling the appropriate methods of
*   the callback.
*
*	For example, ISupplierCallback is a callback interface for suppliers.
*	Any supplier implementation written in Java must implement
*   edu.cmu.aura.supplier.ISupplierCallback
*
*	Similarly, any EnvironmentManager written in Java must
*	implement edu.cmu.aura.em.IEnvironmentManagerCallback
*
*	Methods in the callback interfaces will be called upon the
*	arrival of a message on the wire.  Actual obtaining of the
*	messages from the wire + unpacking are done by helper
*	classes called Connectors and Ports.
*
*   Notice that a Callback can be specified for handling of all methods,
*   or on a per-method basis.  When an object registers with a connector
*   object as a callback handler, this object acts as a catch-all for
*   all callback methods.

*   During runtime, specific callback handlers for specific methods can be
*   specified.  In that case, the general callback will not be called.
*
*   Per-method callbacks are avaiable for supplier
*   connectors, and are enabled by connector sub-interfaces.
*
*	@see edu.cmu.aura.connector.IListener
**/

import java.io.IOException;

public interface IConnector
{
	public abstract void errorSending(	String message,
										String remoteHost,
										int remotePort,
										IOException exception);

	public void receive (String message);
}
