/*
  File: IMessageQueue.java

  History:
  Date       	Who				What
  May_19_2003  	poladian		Created public version
*/

package messaging;

/**
 *  This is a basic queue / de-queue interface to be provided by all Aura
 *  Connectors.  The IListener classes will require an  instance of a
 *  IMessageQueue, where they will queue messages read.
 *
 *  @see edu.cmu.aura.connector.IListener
 **/

interface IMessageQueue
{

/**
 *  Queue the given String message.  This can potentially be a blocking call.
 *  If the underlying queue has reached a capacity, then calling thread will
 *  block, waiting until a slot is available.
 *  @param strMessage message to be placed on queue
 *  @return integer code indicating the outcome of the operation
 *  0 for success and anything else for failure
 *  @throws java.lang.InterruptedException if the calling thread receives
 *  and interrupt message while in waiting state
 **/
	void queueMessage(MessageWrapper strMessage)
					throws java.lang.InterruptedException;

/**
 *  Get next message off the queue.  This is a blocking call.
 *  If there are no pending messages, this call will block.
 *
 *  @return the message
 *  @return the message
 *  @throws java.lang.InterruptedException if the calling thread receives
 *  an interrupt message while in waiting state
 **/
	MessageWrapper getNextMessage()
					throws java.lang.InterruptedException;

}