/*
  File: YieldingSocketProcessor.java

  History:
  Date       	Who				What
  May_19_2003  	poladian		Created public version
*/

package messaging;
import java.io.*;
import java.net.*;


/**
 *  This class implements the run() method in java.lang.Runnable
 *  as a loop in which incoming socket messages are procesed.

 *  this class is internally used by PooledListener.

 *  The loop in the run method is run as long as they are
 *  message in the socket, but not more than a specified number of times.

 *  current default for thta is 5.  because of the yielding, cooperative
 *  nature of this class, we call it Yielding processor.

 *
 *  Read messages are queued on a IMessageQueue.  The MessageQueue used
 *  is the same one that is passed into the IListener object during
 *  instantiation.  Messages are queued in-order of arrival.  However,
 *  since the same queue might be multi-plexed by other isntances of
 *  the same class, there is no guarantee as to the global order.

 * @see edu.cmu.aura.connector.PooledListener
 * @see edu.cmu.aura.connector.IMessageQueue
 **/

class YieldingSocketReader
				implements java.lang.Runnable
{


	protected Socket 				m_socket;
	protected IMessageQueue		m_MessageQueue;

	protected Reader 				m_reader;
	protected Writer				m_writer;


	protected InputStream			m_inStream;
	protected OutputStream			m_outStream;


	protected boolean				m_bIsScheduled;
	protected boolean 				m_bAllowedToDump = false;

	protected char []				m_cBuf;

	protected int					m_intId;


	protected static int			GlobalId = 0;

	protected synchronized static 	int assignId()
	{
		GlobalId++;
		return GlobalId;
	}

	public YieldingSocketReader (
				String name,
				Socket clientSoc,
				IMessageQueue messageQueue)
				throws java.io.IOException

	{
	//	super( name + " - YieldingSocketReader");
		m_intId = assignId();
		m_socket  = clientSoc;
		m_MessageQueue = messageQueue;

		m_inStream = m_socket.getInputStream();
		m_outStream = m_socket.getOutputStream();

		m_reader = new InputStreamReader (m_socket.getInputStream());
		m_writer = new OutputStreamWriter (m_socket.getOutputStream());
		m_cBuf = new char[4096];
	}

	 int getId()
	{
		return m_intId;
	}

	 void schedule()
	{
		m_bIsScheduled = true;
	}

	void unSchedule()
	{
		m_bIsScheduled = false;
	}

	boolean isScheduled()
	{
		return m_bIsScheduled;
	}

	public void run()
	{
		boolean flag = false;
		String strMessage = null;
		try
		{
			for( int i =0; i < 5; i ++ )
			{
				flag = false;
				strMessage = SocketProtocolMessageReader.readMessage(m_reader);
				flag = true;
				MessageWrapper wrap = new MessageWrapper();
				wrap._message = strMessage;


				m_MessageQueue.queueMessage(wrap);
				flag = false;

				if ( m_inStream.available() == 0 )
					break;
			}
		}
		catch (java.lang.InterruptedException interrupted)
		{
			interrupted.printStackTrace();

			System.out.println("YieldingSocketProcessor :: run interrupted.  Aborting run loop, closing socket, dumping message: \n" + strMessage);
			prepareToDump();
			Thread.currentThread().interrupt();
		}
		catch (java.io.IOException ioExc)
		{
			//ioExc.printStackTrace();
			prepareToDump();
		}
		catch (java.lang.RuntimeException runTimeExc)
		{
			System.out.println("Offending socket, caused runtime exception, closing");
			runTimeExc.printStackTrace();
			prepareToDump();
		}
		unSchedule();
	}

	 void prepareToDump()
	{
		try
		{
			m_inStream.close();
			m_outStream.close();
			m_socket.close();
		}
		catch (java.io.IOException exc)
		{
			exc.printStackTrace();
		}
		m_bAllowedToDump = true;
	}

	 boolean canDump()
	{
		return m_bAllowedToDump;
	}
}