package util;


/**
 *
 *	This class implements a sleep method, similar to
 *	Thread.sleep(), but catches InterruptedExceptions.
 *
 *	Upon interruption by such an exception, it breaks
 *	the sleep, thus conforming to the behavior of the
 *	Thread.interrupt().  At this point, no propagation of
 *	the interrupt is done, see the xxx comment below.
 *
 *	Sleep is not guaranteed to be exactly the prescribed
 *	number of milliseconds.
 *
*/



public class AuraSleep
{
/**
 *
 *	Sleep for prescribed number of milliseconds
 *  @param millis time to sleep
 */
 public static void sleep(int millis)
	{
		long lBedTime = System.currentTimeMillis();

		int remainingTime = millis;

		while (true)
		{
			try
			{
				Thread.sleep(remainingTime);
			}
			catch (InterruptedException iExc)
			{
				/* xxx is this the right behavior?
				should we do Thread.interrupt()?  */
				break;
			}
			catch(Exception exc)
			{

			}
			long lWakeTime = System.currentTimeMillis();
			int sleptFor = (int) (lWakeTime - lBedTime);
			if ( ( sleptFor - millis) >= 0 )
				break;

			remainingTime = millis - sleptFor;
		}
	}
}