/*
 * Created on Feb 11, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package util.xml;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;

import util.xml.nanoxml.XMLElement;
import util.xml.nanoxml.XMLParseException;


/**
 * The purpose of this class is to wrap the particular XML parser being used
 * 
 * @author jpsousa
 */
public class XmlElement {
	private XMLElement _xmlElement;

	/**
	 * @param elements
	 *            Array list of XmlElement(s) to be indexed
	 * @param att
	 *            to index by
	 * @return HashMap of the elements indexed by att
	 */
	public static HashMap indexByAtt(ArrayList elements, String att) {
		int count = elements.size();
		HashMap map = new HashMap(count);
		for (int i = 0; i < count; i++) {
			XmlElement element = (XmlElement) elements.get(i);
			map.put(element.getAttribute(att, null), element);
		}
		return map;
	}

	/**
	 * @param elements
	 * @param attName
	 *            name of attribute to search (not null)
	 * @param attVal
	 *            searched value (not null)
	 * @return first element whose attribute attName has value attValue, or null
	 */
	public static XmlElement findByAtt(Collection elements, String attName, String attVal) {
		Iterator elementIt = elements.iterator();
		while (elementIt.hasNext()) {
			XmlElement element = (XmlElement) elementIt.next();
			if (attVal.equals(element.getAttribute(attName, null)))
				return element;
		}
		return null;
	}

	/**
	 * Creates a new element from scratch. Must at least call setName() on it to
	 * obtain a valid XML element. All setters and getters are case preserving.
	 * 
	 * @see util.xml.XmlElement#setName(String name)
	 */
	public XmlElement() {
		_xmlElement = new XMLElement(new Hashtable(), false, false);
	}

	/**
	 * The new element contains the (case preserving) results of parsing the
	 * string.
	 * 
	 * @param text
	 * @throws XmlParseException
	 */
	public XmlElement(String text) throws XmlParseException {
		this();
		try {
			_xmlElement.parseString(text);
		} catch (XMLParseException e) {
			throw new XmlParseException(e);
		}
	}

	/**
	 * The new element contains the (case preserving) results of parsing the
	 * file.
	 * 
	 * @param reader
	 * @throws XmlParseException
	 * @throws IOException
	 */
	public XmlElement(FileReader reader) throws XmlParseException, IOException {

		this();
		try {
			_xmlElement.parseFromReader(reader);
		} catch (XMLParseException e) {
			throw new XmlParseException(e);
		}
	}

	private XmlElement(XMLElement alterEgo) {
		_xmlElement = alterEgo;
	}
	private XMLElement alterEgo() {
		return _xmlElement;
	}

	/**
	 * @return a fresh (complete) copy of the original
	 */
	public XmlElement xerox() {
		XmlElement arnie = new XmlElement();
		arnie.alterEgo().parseString(_xmlElement.toString());
		return arnie;
	}

	/**
	 * @param name
	 *            must not be null
	 */
	public void setName(String name) {
		_xmlElement.setName(name);
	}
	public String getName() {
		return _xmlElement.getName();
	}

	/**
	 * Sets a new attribute, or overides the previous value
	 * 
	 * @param name
	 *            must not be null
	 * @param value,
	 *            if null, the attribute is removed
	 */
	public void setAttribute(String name, String value) {
		if (value == null)
			_xmlElement.removeAttribute(name);
		else
			_xmlElement.setAttribute(name, value);
	}
	public String getAttribute(String name, String defaultValue) {
		return _xmlElement.getStringAttribute(name, defaultValue);
	}
	public void removeAttribute(String name) {
		_xmlElement.removeAttribute(name);
	}

	public void addChild(XmlElement child) {
		if (child != null)
			_xmlElement.addChild(child.alterEgo());
	}

	/**
	 * @param children
	 *            ArrayList of XmlElement
	 */
	public void addChildren(ArrayList children) {
		int count = children.size();
		for (int i = 0; i < count; i++)
			_xmlElement.addChild(((XmlElement) children.get(i)).alterEgo());
	}

	public int countChildren() {
		return _xmlElement.countChildren();
	}

	/**
	 * @return a fresh ArrayList with the children. Modifying this list does not
	 *         change the XmlElement (modifying a child in the list does).
	 */
	public ArrayList getChildren() {
		ArrayList allChildren = new ArrayList();
		Enumeration children = _xmlElement.enumerateChildren();
		while (children.hasMoreElements()) {
			XMLElement child = (XMLElement) children.nextElement();
			allChildren.add(new XmlElement(child));
		}
		return allChildren;
	}

	/**
	 * @param name
	 *            must not be null
	 * @return the first child for which name.equals(child.getName()), or null
	 *         if no such child exists.
	 */
	public XmlElement getFirstChild(String name) {
		Enumeration children = _xmlElement.enumerateChildren();

		while (children.hasMoreElements()) {
			XMLElement child = (XMLElement) children.nextElement();
			if (name.equals(child.getName())) {
				return new XmlElement(child);
			}
		}
		return null;
	}

	/**
	 * @param name
	 *            must not be null
	 * @return a fresh ArrayList with the children for which
	 *         name.equals(child.getName()). Modifying this list does not change
	 *         the XmlElement (modifying a child in the list does).
	 */
	public ArrayList getChildren(String name) {
		ArrayList selectedChildren = new ArrayList();
		Enumeration children = _xmlElement.enumerateChildren();
		while (children.hasMoreElements()) {
			XMLElement child = (XMLElement) children.nextElement();
			if (name.equals(child.getName())) {
				selectedChildren.add(new XmlElement(child));
			}
		}
		return selectedChildren;
	}

	public void removeChild(XmlElement child) {
		if (child != null)
			_xmlElement.removeChild(child.alterEgo());
	}

	/**
	 * @param children
	 *            collection of (XmlElement) children to be removed
	 */
	public void removeChildren(Collection children) {
		Iterator childIterator = children.iterator();
		while (childIterator.hasNext()) {
			XmlElement child = (XmlElement) childIterator.next();
			removeChild(child);
		}
	}

	/**
	 * Sets the PCData for the element. Passing <code>null</code> resets the
	 * PCData.
	 * 
	 * @param content
	 */
	public void setContent(String content) {
		_xmlElement.setContent(content);
	}

	public String getContent(String defaultValue) {
		String content = _xmlElement.getContent();
		if (content == null)
			return defaultValue;
		else
			return content;
	}

	/**
	 * @param childName
	 * @return ArrayList of the PCData for the children named childName
	 */
	public ArrayList getChildrenContent(String childName) {
		ArrayList children = getChildren(childName);
		int childCount = children.size();
		ArrayList contents = new ArrayList(childCount);
		for (int i = 0; i < childCount; i++) {
			String content = ((XmlElement) children.get(i)).getContent("");
			contents.add(i, content);
		}
		return contents;
	}

	public String toString() {
		return _xmlElement.toString();
	}
}